unit CalcSigDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.Dialogs, Vcl.ComCtrls,
  CalcSigMain, CbFunctions;

const
  // error codes
  errOK         = 0;
  errFileOpen   = 2;
  errNotFound   = 14; // source file not found
  errUserBreak  = $80; // process stopped by user

type
  TCalcSigDialog = class(TForm)
    OKBtn: TBitBtn;
    edFile: TLabeledEdit;
    edMd5: TLabeledEdit;
    sbCopyMD5: TSpeedButton;
    edSha: TLabeledEdit;
    sbCopySHA: TSpeedButton;
    cbMd5D: TCheckBox;
    cbShaD: TCheckBox;
    btnProcess: TBitBtn;
    pbMD5: TProgressBar;
    edSize: TLabeledEdit;
    pbSHA: TProgressBar;
    cbWriteFile: TCheckBox;
    edSigFile: TLabeledEdit;
    laVers: TLabel;
    cbSHA2D: TCheckBox;
    edSha2: TLabeledEdit;
    sbCopySHA2: TSpeedButton;
    pbSHA2: TProgressBar;
    procedure FormCreate(Sender: TObject);
    procedure btnProcessClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure sbCopyMD5Click(Sender: TObject);
    procedure sbCopySHAClick(Sender: TObject);
    procedure cbWriteFileClick(Sender: TObject);
    procedure sbCopySHA2Click(Sender: TObject);
  private
    { Private-Deklarationen }
    Stopped : boolean;
    SigThread : array [TSigType] of TDigiSigThread;
    Total,ProgMD5,ProgSHA,ProgSHA2 : int64;
    procedure ShowProgressMD5(ProgressType : TProgressType; Value : int64);
    procedure ShowProgressSHA(ProgressType : TProgressType; Value : int64);
    procedure ShowProgressSHA2(ProgressType : TProgressType; Value : int64);
    procedure ProcessFile;
  public
    { Public-Deklarationen }
    AutoCalc : boolean;
  end;

function GetCopyErrMsg (AError : integer) : string;

var
  CalcSigDialog: TCalcSigDialog;

implementation

{$R *.dfm}

uses GnuGetText, FileUtils, FileConsts, WinUtils, StringUtils;

{ ------------------------------------------------------------------- }
function GetCopyErrMsg (AError : integer) : string;
begin
  if AError=errOK then Result:=''
  else begin
    Result:=rsError;
    case AError of
    errFileOpen   : Result:=Result+rsFileOpen; //Could not open file
    errNotFound   : Result:=Result+rsNotFound;   // File not found
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TCalcSigDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self);
  AutoCalc:=false;
  end;

procedure TCalcSigDialog.FormShow(Sender: TObject);
begin
  Stopped:=true;
  if AutoCalc then ProcessFile;
  AutoCalc:=false;
  end;

procedure TCalcSigDialog.cbWriteFileClick(Sender: TObject);
begin
  edSigFile.Enabled:=cbWriteFile.Checked;
  end;

procedure TCalcSigDialog.sbCopyMD5Click(Sender: TObject);
begin
  with edMd5 do begin
    SelectAll; CopyToClipBoard; SelLength:=0;
    end;
  end;

procedure TCalcSigDialog.sbCopySHAClick(Sender: TObject);
begin
  with edSHA do begin
    SelectAll; CopyToClipBoard; SelLength:=0;
    end;
  end;

procedure TCalcSigDialog.sbCopySHA2Click(Sender: TObject);
begin
  with edSHA2 do begin
    SelectAll; CopyToClipBoard; SelLength:=0;
    end;
  end;

procedure TCalcSigDialog.ShowProgressMD5(ProgressType : TProgressType; Value : int64);
begin
  if ProgressType=ptInc then begin
    inc(ProgMD5,Value);
    pbMD5.Position:=round(ProgMD5/Total*100);
    end
  else if ProgressType=ptStart then pbMD5.Position:=1;
  end;

procedure TCalcSigDialog.ShowProgressSHA(ProgressType : TProgressType; Value : int64);
begin
  if ProgressType=ptInc then begin
    inc(ProgSHA,Value);
    pbSHA.Position:=round(ProgSHA/Total*100);
    end
  else if ProgressType=ptStart then pbSHA.Position:=1;
  end;

procedure TCalcSigDialog.ShowProgressSHA2(ProgressType : TProgressType; Value : int64);
begin
  if ProgressType=ptInc then begin
    inc(ProgSHA2,Value);
    pbSHA2.Position:=round(ProgSHA2/Total*100);
    end
  else if ProgressType=ptStart then pbSHA2.Position:=1;
  end;

procedure TCalcSigDialog.btnProcessClick(Sender: TObject);
begin
  ProcessFile;
  end;

procedure TCalcSigDialog.ProcessFile;
var
  AllDone : boolean;
  oText   : TextFile;
begin
  if not Stopped then begin
    Stopped:=true; Exit;
    end;
  Stopped:=false;
  pbMD5.Position:=0; pbSHA.Position:=0; pbSHA2.Position:=0;
  Total:=LongFileSize(edFile.Text);
  ProgMD5:=0; ProgSHA:=0; ProgSHA2:=0;
  with btnProcess do begin
    Glyph:=nil;
    MainForm.ImageList.GetBitmap(1,Glyph);
    Caption:=_('Stop processing file');
    end;
  OKBtn.Enabled:=false;
  if Total>0 then begin
    if cbMd5D.Checked then begin
      SigThread[stMD5]:=TMD5Thread.Create(edFile.Text,true);
      with SigThread[stMD5] do begin
        Priority:=tpNormal;
        OnProgress:=ShowProgressMD5;
        Resume;
        end;
      end
    else SigThread[stMD5]:=nil;
    if cbSHAD.Checked then begin
      SigThread[stSHA1]:=TSHA1Thread.Create(edFile.Text,true);
      with SigThread[stSHA1] do begin
        Priority:=tpNormal;
        OnProgress:=ShowProgressSHA;
        Resume;
        end;
      end
    else SigThread[stSHA1]:=nil;
    if cbSHA2D.Checked then begin
      SigThread[stSHA256]:=TSHA256Thread.Create(edFile.Text,true);
      with SigThread[stSHA256] do begin
        Priority:=tpNormal;
        OnProgress:=ShowProgressSHA2;
        Resume;
        end;
      end
    else SigThread[stSHA256]:=nil;
    AllDone:=true;
    repeat
      Sleep(0);
      Application.ProcessMessages;
      if Stopped then begin
        if assigned(SigThread[stMD5]) then SigThread[stMD5].CancelThread;
        if assigned(SigThread[stSHA1]) then SigThread[stSHA1].CancelThread;
        if assigned(SigThread[stSHA256]) then SigThread[stSHA256].CancelThread;
        end;
      if assigned(SigThread[stMD5]) then AllDone:=SigThread[stMD5].Done;
      if assigned(SigThread[stSHA1]) then AllDone:=SigThread[stSHA1].Done;
      if assigned(SigThread[stSHA256]) then AllDone:=SigThread[stSHA256].Done;
      until AllDone;
    if assigned(SigThread[stMD5]) then with SigThread[stMD5] do begin
      if ErrorCode=errOK then edMd5.Text:=UpperCase(Signature)
      else edMd5.Text:=GetCopyErrMsg(ErrorCode);
      FreeAndNil(SigThread[stMD5]);
      end
    else edMd5.Text:='';
    if assigned(SigThread[stSHA1]) then with SigThread[stSHA1] do begin
      if ErrorCode=errOK then edSHA.Text:=UpperCase(Signature)
      else edSHA.Text:=GetCopyErrMsg(ErrorCode);
      FreeAndNil(SigThread[stSHA1]);
      end
    else edSHA.Text:='';
    if assigned(SigThread[stSHA256]) then with SigThread[stSHA256] do begin
      if ErrorCode=errOK then edSHA2.Text:=UpperCase(Signature)
      else edSHA2.Text:=GetCopyErrMsg(ErrorCode);
      FreeAndNil(SigThread[stSHA256]);
      end
    else edSHA2.Text:='';
    end;
  with pbMd5 do if Position>0 then begin
    Position:=99; Position:=100;
    end;
  with pbSHA do if Position>0 then begin
    Position:=99; Position:=100;
    end;
  with pbSHA2 do if Position>0 then begin
    Position:=99; Position:=100;
    end;
  if cbWriteFile.Checked then begin
    AssignFile(oText,ExtractFilepath(edFile.Text)+edSigFile.Text);
    rewrite(oText);
    writeln (oText,ExtractFilename(edFile.Text)+' ('+IntToStr(LongFileSize(edFile.Text))+' byte)');
    if cbMd5D.Checked then
      writeln (oText,AddSp(SigNames[stMd5],7)+': ',edMd5.Text);
    if cbSHAD.Checked then
      writeln (oText,AddSp(SigNames[stSHA1],7)+': ',edSHA.Text);
    if cbSHA2D.Checked then
      writeln (oText,AddSp(SigNames[stSHA256],7)+': ',edSHA2.Text);
    CloseFile(oText);
    end;
  with btnProcess do begin
    Glyph:=nil;
    MainForm.ImageList.GetBitmap(0,Glyph);
    Caption:=_('Start processing file');
    end;
  OKBtn.Enabled:=true;
  Stopped:=true;
  end;

end.
